@@ -36,7 +36,7 @@ avi_delete (AviEdit    *avi_edit,
             const gint  samples)
 {
   gint i, j = 0, samples_n;
-  AviFile *p, *q, *r;
+  AviFile *p, *q;
 
   /* ja:スタート位置とサンプル数の調整 */
   if (!avi_edit || start < 0)
@@ -83,6 +83,8 @@ avi_delete (AviEdit    *avi_edit,
     }
   while (p != q)
     {
+      AviFile *r;
+
       if (avi_edit->file == p)
         avi_edit->file = avi_edit->file->next;
       if (p->prev)
@@ -111,7 +113,6 @@ avi_copy (AviEdit    *avi_edit,
 {
   gint start_n, samples_n;
   AviEdit *avi_copy;
-  AviFile *avi_file;
 
   /* ja:スタート位置とサンプル数の調整 */
   if (!avi_edit || start < 0)
@@ -142,6 +143,8 @@ avi_copy (AviEdit    *avi_edit,
   start_n = start;
   while (samples_n > 0)
     {
+      AviFile *avi_file;
+
       /* ja:コピー */
       avi_file = g_malloc (sizeof (AviFile));
 
@@ -198,7 +201,7 @@ avi_paste (AviEdit    *avi_dst,
            const gint  pos,
            AviEdit    *avi_src)
 {
-  gint i, length;
+  gint i;
   AviFile *p, *q;
 
   /* ja:違う種類または範囲外ならばエラー */
@@ -207,6 +210,8 @@ avi_paste (AviEdit    *avi_dst,
     return FALSE;
   if (avi_type (avi_dst) == AVI_STREAM_AUDIO)
     {
+      gint length;
+
       /* ja:オーディオ */
       length = wf_header_bytes (avi_dst->wfx);
       if (length != wf_header_bytes (avi_src->wfx)
@@ -259,7 +264,7 @@ avi_paste (AviEdit    *avi_dst,
   g_free (avi_src->wfx);
   g_free (avi_src);
   /* ja:最適化 */
-  if (p && p->name && p->next->name && !g_strcmp (p->name, p->next->name)
+  if (p && p->name && p->next->name && g_strcmp (p->name, p->next->name) == 0
                                     && p->start + p->length == p->next->start)
     {
       p->length += p->next->length;
@@ -275,7 +280,7 @@ avi_paste (AviEdit    *avi_dst,
         p->next->prev=p->prev;
       avi_release_file (p);
     }
-  if (q && q->name && q->prev->name && !g_strcmp (q->name, q->prev->name)
+  if (q && q->name && q->prev->name && g_strcmp (q->name, q->prev->name) == 0
                             && q->prev->start + q->prev->length == q->start)
     {
       if (avi_dst->file == q)
