@@ -153,7 +153,6 @@ avi_open (const gchar *file,
           const gint   param)
 {
   AviEdit *avi_edit;
-  IcmObject *icm_object;
 
   if (!file)
     return NULL;
@@ -176,7 +175,7 @@ avi_open (const gchar *file,
                             || avi_edit->file->handler == BI_COMP_BITFIELDS)
         {
           avi_edit->file->handler = AVI_STREAM_COMP_DIB;
-          avi_edit->bmih=g_memdup (avi_edit->file->bmih,
+          avi_edit->bmih = g_memdup (avi_edit->file->bmih,
                                     bm_header_bytes (avi_edit->file->bmih));
           if (avi_get_compression (avi_edit) == AVI_STREAM_COMP_DIB)
             bmih_set_compression (avi_edit->bmih, BI_COMP_RGB);
@@ -186,6 +185,8 @@ avi_open (const gchar *file,
         }
       else
         {
+          IcmObject *icm_object;
+
           /* ja:圧縮 */
           icm_object = icm_open (avi_edit->file->handler,
                                                         ICM_MODE_DECOMPRESS);
@@ -238,12 +239,13 @@ gboolean
 avi_release (AviEdit *avi_edit)
 {
   gint i;
-  AviFile *avi_file;
 
   if (!avi_edit)
     return FALSE;
-  while (!avi_edit->file)
+  while (avi_edit->file)
     {
+      AviFile *avi_file;
+
       avi_file = avi_edit->file->next;
       avi_release_file (avi_edit->file);
       avi_edit->file = avi_file;
