@@ -37,8 +37,8 @@ avi_at_scenario (AviEdit    *avi_edit,
                  const gint  start,
                  const gint  samples)
 {
-  gchar *text, *scenario = NULL;
-  gint i, j, leng, length = 0, size, start_n, samples_n;
+  gchar *scenario = NULL;
+  gint length = 0, start_n, samples_n;
 
   /* ja:スタート位置とサンプル数の調整 */
   if (start < 0)
@@ -50,6 +50,9 @@ avi_at_scenario (AviEdit    *avi_edit,
   start_n = start;
   while (samples_n > 0)
     {
+      gchar *text;
+      gint leng;
+
       avi_get_file (avi_edit, start_n);/* ja:AVIファイルを求める */
       if (avi_edit->file->name)
         {
@@ -64,10 +67,11 @@ avi_at_scenario (AviEdit    *avi_edit,
         }
       else if (avi_edit->file->data && avi_edit->file->bmih)
         {
+          gint i, j = 0, size;
+
           /* ja:メモリ内ビットマップ */
           text = g_strdup_printf ("%d,bitmap,%d,\\\n",
                                 stream, bm_all_bytes (avi_edit->file->bmih));
-          j = 0;
           leng = g_strlen (text);
           size = bm_header_bytes (avi_edit->file->bmih);
           for (i = 0; i < size; i++)
@@ -132,8 +136,7 @@ avi_to_scenario (AviEdit        *avi_edit[],
                  const gboolean  interleave)
 {
   gchar *text, *scenario = NULL;
-  gint i, leng, length = 0, streams, current = 0, *pos, samples;
-  gint timer = 0, times;
+  gint i, leng, length = 0;
 
   if (!avi_edit)
     return NULL;
@@ -200,6 +203,8 @@ avi_to_scenario (AviEdit        *avi_edit[],
   /* ja:本体 */
   if (interleave)
     {
+      gint current = 0, *pos, streams, timer = 0, times;
+
       /* ja:インターリーブ */
       for (streams = 0; avi_edit[streams]; streams++);
       /* ja:最も1秒間のフレーム数が少ないストリームを探す */
@@ -242,6 +247,8 @@ avi_to_scenario (AviEdit        *avi_edit[],
           for (i = 0; i < streams; i++)
             if (pos[i] < avi_length (avi_edit[i]))
               {
+                gint samples;
+
                 samples = current > 0 ? i == current ? 1 : avi_time_to_sample
                         (avi_edit[i], avi_sample_to_time (avi_edit[current],
                                                     pos[current] + 1)) - pos[i]
@@ -297,13 +304,10 @@ avi_to_scenario (AviEdit        *avi_edit[],
 AviEdit **
 avi_from_scenario (const gchar *scenario)
 {
-  gboolean quote[2];
-  gchar **lines, **cmd, *text;
-  gint i, j, stream, count = 0, leng, param, start, length, value, size;
-  guint32 type;
+  gchar **lines, *text;
+  gint i, j, count = 0;
   AviEdit **avi_edit = NULL;
   AviFile *avi_file;
-  IcmObject *icm_object;
 
   if (!scenario)
     return NULL;
@@ -318,6 +322,10 @@ avi_from_scenario (const gchar *scenario)
   g_free (text);
   for (i = 0; lines[i]; i++)
     {
+      gboolean quote[2];
+      gchar **cmd;
+      gint stream;
+
       /* ja:コメント部分を削除,カンマを改行にする */
       quote[0] = quote[1] = FALSE;
       for (j = 0; (lines[i])[j] != '\0'; j++)
@@ -345,6 +353,8 @@ avi_from_scenario (const gchar *scenario)
       cmd = g_strsplit (lines[i], "\n", G_MAXINT);
       for (j = 0; cmd[j]; j++)
         {
+          gint leng;
+
           g_strstrip (cmd[j]);
           leng = g_strlen (cmd[j]) - 1;
           if (leng >= 1 && (((cmd[j])[0] == '\'' && (cmd[j])[leng] == '\'')
@@ -357,6 +367,8 @@ avi_from_scenario (const gchar *scenario)
       if (cmd[0] && cmd[1]
                 && misc_str_to_val (&stream, cmd[0], 10, TRUE) && stream >= 0)
         {
+          gint length, size, value;
+
           if (count <= stream)/* ja:拡張 */
             {
               avi_edit = g_realloc (avi_edit,
@@ -367,38 +379,38 @@ avi_from_scenario (const gchar *scenario)
             }
           if (!avi_edit[stream])/* ja:新規 */
             avi_edit[stream] = g_malloc0 (sizeof (AviEdit));
-          if (!g_strcasecmp (cmd[1], "type") && cmd[2])
+          if (g_strcasecmp (cmd[1], "type") == 0 && cmd[2])
             {
-              if (!g_strcasecmp (cmd[2], "audio"))
+              if (g_strcasecmp (cmd[2], "audio") == 0)
                 avi_edit[stream]->type = AVI_STREAM_AUDIO;
-              else if (!g_strcasecmp (cmd[2], "video"))
+              else if (g_strcasecmp (cmd[2], "video") == 0)
                 avi_edit[stream]->type = AVI_STREAM_VIDEO;
             }
-          else if (!g_strcasecmp (cmd[1], "rate") && cmd[2])
+          else if (g_strcasecmp (cmd[1], "rate") == 0&& cmd[2])
             {
               if (misc_str_to_val (&value, cmd[2], 10, FALSE))
                 avi_set_rate (avi_edit[stream], value);
             }
-          else if (!g_strcasecmp (cmd[1], "scale") && cmd[2])
+          else if (g_strcasecmp (cmd[1], "scale") == 0 && cmd[2])
             {
               if (misc_str_to_val (&value, cmd[2], 10, FALSE))
                 avi_set_scale (avi_edit[stream], value);
             }
-          else if (!g_strcasecmp (cmd[1], "priority") && cmd[2])
+          else if (g_strcasecmp (cmd[1], "priority") == 0 && cmd[2])
             {
               if (misc_str_to_val (&value, cmd[2], 10, FALSE))
                 avi_set_priority (avi_edit[stream], value);
             }
-          else if (!g_strcasecmp (cmd[1], "language") && cmd[2])
+          else if (g_strcasecmp (cmd[1], "language") == 0 && cmd[2])
             {
               if (misc_str_to_val (&value, cmd[2], 16, FALSE))
                 avi_set_language (avi_edit[stream], value);
             }
-          else if (!g_strcasecmp (cmd[1], "name") && cmd[2])
+          else if (g_strcasecmp (cmd[1], "name") == 0 && cmd[2])
             {
               avi_set_name (avi_edit[stream], cmd[2]);
             }
-          else if (!g_strcasecmp (cmd[1], "bmih"))
+          else if (g_strcasecmp (cmd[1], "bmih") == 0)
             {
               for (j = 2; cmd[j]; j++);
                 if (j >= 7)
@@ -456,8 +468,11 @@ avi_from_scenario (const gchar *scenario)
                       }
                   }
             }
-          else if (!g_strcasecmp (cmd[1], "file") && cmd[2])
+          else if (g_strcasecmp (cmd[1], "file") == 0 && cmd[2])
             {
+              gint param;
+              guint32 type;
+
               if (!cmd[3] || !misc_str_to_val (&param, cmd[3], 10, FALSE))
                 param = 0;
               avi_file = avi_open_file (cmd[2], param, &type,
@@ -482,6 +497,8 @@ avi_from_scenario (const gchar *scenario)
                 }
               if (avi_file)
                 {
+                  gint start;
+
                   if (cmd[3] || cmd[4]
                             || !misc_str_to_val (&start, cmd[4], 10, TRUE))
                     start = -1;
@@ -500,8 +517,8 @@ avi_from_scenario (const gchar *scenario)
                       /* ja:先頭 */
                       avi_edit[stream]->file = avi_file;
                     }
-                  else if (!g_strcmp (avi_edit[stream]->file->name,
-                                                                avi_file->name)
+                  else if (g_strcmp (avi_edit[stream]->file->name,
+                                                        avi_file->name) == 0
                       && avi_edit[stream]->file->start
                         + avi_edit[stream]->file->length == avi_file->start)
                     {
@@ -518,7 +535,7 @@ avi_from_scenario (const gchar *scenario)
                     }
                 }
             }
-          else if (!g_strcasecmp (cmd[1], "bitmap") && cmd[2]
+          else if (g_strcasecmp (cmd[1], "bitmap") == 0 && cmd[2]
                                 && misc_str_to_val (&size, cmd[2], 10, FALSE)
                                 && size > BMIH_SIZE)
             {
@@ -526,7 +543,7 @@ avi_from_scenario (const gchar *scenario)
               avi_file->data = NULL;            /* ja:メモリ */
               /* ja:ヘッタを読み込む */
               avi_file->bmih = g_malloc (BMIH_SIZE);
-              for (j = 0; j < BMIH_SIZE && cmd[j + 3] != NULL; j++)
+              for (j = 0; j < BMIH_SIZE && cmd[j + 3]; j++)
                 {
                   if (!misc_str_to_val (&value, cmd[j + 3], 16, FALSE))
                     break;
@@ -632,7 +649,7 @@ avi_from_scenario (const gchar *scenario)
                 avi_edit[i]->file=avi_edit[i]->file->prev;
               for (avi_file = avi_edit[i]->file; avi_file;
                                                     avi_file = avi_file->next)
-                avi_edit[i]->length+=avi_file->length;
+                avi_edit[i]->length += avi_file->length;
             }
           if (avi_length (avi_edit[i]) <= 0)
             {
@@ -641,7 +658,7 @@ avi_from_scenario (const gchar *scenario)
             }
           /* ja:展開後のフォーマット */
           if (avi_edit[i] && !avi_edit[i]->bmih
-                                && avi_type(avi_edit[i]) == AVI_STREAM_VIDEO)
+                                && avi_type (avi_edit[i]) == AVI_STREAM_VIDEO)
             {
               if (avi_edit[i]->file->handler == AVI_STREAM_COMP_DIB)
                 {
@@ -655,6 +672,8 @@ avi_from_scenario (const gchar *scenario)
                 }
               else
                 {
+                  IcmObject *icm_object;
+
                   /* ja:圧縮 */
                   icm_object = icm_open (avi_edit[i]->file->handler,
                                                         ICM_MODE_DECOMPRESS);
@@ -684,7 +703,7 @@ avi_from_scenario (const gchar *scenario)
                 }
             }
           else if (avi_edit[i] && !avi_edit[i]->wfx
-                                && avi_type(avi_edit[i]) == AVI_STREAM_AUDIO)
+                                && avi_type (avi_edit[i]) == AVI_STREAM_AUDIO)
             {
               if (wfx_get_format_tag (avi_edit[i]->file->wfx)
                                                             == WAVE_FORMAT_PCM)
