@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -90,7 +90,7 @@ typedef struct { guint8 dummy[RGBQUAD_SIZE]; } RgbQuad;
 
 /******************************************************************************
 *                                                                             *
-* ja:\?a??a????\?￣\?                                               *
+* ja:ビットマップ構造体マクロ                                                 *
 *                                                                             *
 ******************************************************************************/
 #define bmfh_get_type(bmfh) GUINT16_FROM_LE(*(guint16 *)(bmfh))
@@ -159,326 +159,326 @@ typedef struct { guint8 dummy[RGBQUAD_SIZE]; } RgbQuad;
                         *((guint8 *)(rgb)+(index)*4+RGB_RED)=(red),         \
                         *((guint8 *)(rgb)+(index)*4+RGB_RESERVED)=(reserved))
 #define rgb_get(bmih,index,color)                                           \
-                            rgbquad_get((guint8 *)(bmih)+BMIH_SIZE,index,color)
+            rgbquad_get((guint8 *)(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE),\
+                                                                index,color)
 #define rgb_set(bmih,index,color,value)                                     \
-                    rgbquad_set((guint8 *)(bmih)+BMIH_SIZE,index,color,value)
+            rgbquad_set((guint8 *)(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE),\
+                                                            index,color,value)
 #define rgb_set_all(bmih,index,blue,green,red,reserved) (                   \
-                    rgbquad_set_all((guint8 *)(bmih)+BMIH_SIZE,index,color, \
-                                                    blue,green,red,reserved)
+        rgbquad_set_all((guint8 *)(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE),\
+                                        index,color,blue,green,red,reserved)
 
 
 /******************************************************************************
 *                                                                             *
-* ja:\?a??a??￣\?                                                     *
+* ja:ビットマップマクロ                                                       *
 *                                                                             *
 ******************************************************************************/
-/*  ja:RgbQuad1????ο?μ??e
-          width,?
-         height,1?
-      bit_count,1\?￣\≫\?????\?a?o
-    compression,°μ??|°
-     color_used,?§?o
-            RET,RgbQuad1????ο?                                         */
+/*  ja:RgbQuad構造体の数を求める
+          width,幅
+         height,高さ
+      bit_count,1ピクセルあたりのビット数
+    compression,圧縮形式
+     color_used,色数
+            RET,RgbQuad構造体の数                                           */
 #define bx_color_used(width,height,bit_count,compression,color_used)        \
                 ((color_used)>0?(color_used):((bit_count)<=8?1<<(bit_count):0))
 
 
-/*  ja:RgbQuad1????????uint32?γ?ΥХ?\??μ??e
-          width,?
-         height,1?
-      bit_count,1\?￣\≫\?????\?a?o
-    compression,°μ??|°
-     color_used,?§?o
-            RET,\Х?\??                                                  */
+/*  ja:RgbQuad構造体またはguint32配列のバイト数を求める
+          width,幅
+         height,高さ
+      bit_count,1ピクセルあたりのビット数
+    compression,圧縮形式
+     color_used,色数
+            RET,バイト数                                                    */
 #define bx_color_bytes(width,height,bit_count,compression,color_used)       \
             ((compression)!=BI_COMP_BITFIELDS                               \
             ?bx_color_used(width,height,bit_count,compression,color_used)   \
                                             *RGBQUAD_SIZE:sizeof(guint32)*3)
 
 
-/*  ja:BitmapInfoHeader1?????gbQuad1????????uint32?γ?ΥХ?\??μ??e
-          width,?
-         height,1?
-      bit_count,1\?￣\≫\?????\?a?o
-    compression,°μ??|°
-     color_used,?§?o
-            RET,\Х?\??                                                  */
+/*  ja:BitmapInfoHeader構造体とRgbQuad構造体またはguint32配列のバイト数を求める
+          width,幅
+         height,高さ
+      bit_count,1ピクセルあたりのビット数
+    compression,圧縮形式
+     color_used,色数
+            RET,バイト数                                                    */
 #define bx_header_bytes(width,height,bit_count,compression,color_used)      \
     (bx_color_bytes(width,height,bit_count,compression,color_used)+BMIH_SIZE)
 
 
-/*  ja:1\?\?\Х?\??μ??e
-          width,?
-         height,1?
-      bit_count,1\?￣\≫\?????\?a?o
-    compression,°μ??|°
-     color_used,?§?o
-            RET,\Х?\??                                                  */
+/*  ja:1ラインのバイト数を求める
+          width,幅
+         height,高さ
+      bit_count,1ピクセルあたりのビット数
+    compression,圧縮形式
+     color_used,色数
+            RET,バイト数                                                    */
 #define bx_line_bytes(width,height,bit_count,compression,color_used)        \
                                             ((((width)*(bit_count)+31)&~31)/8)
 
 
-/*  ja:\?￣\≫\?!?\??ΥХ?\??μ??e
-          width,?
-         height,1?
-      bit_count,1\?￣\≫\?????\?a?o
-    compression,°μ??|°
-     color_used,?§?o
-            RET,\Х?\??                                                  */
+/*  ja:ピクセルデータのバイト数を求める
+          width,幅
+         height,高さ
+      bit_count,1ピクセルあたりのビット数
+    compression,圧縮形式
+     color_used,色数
+            RET,バイト数                                                    */
 #define bx_image_bytes(width,height,bit_count,compression,color_used)       \
     (bx_line_bytes(width,height,bit_count,compression,color_used)*(height)) \
 
 
-/*  ja:\?a??a?ΥХ?\??μ??e
-          width,?
-         height,1?
-      bit_count,1\?￣\≫\?????\?a?o
-    compression,°μ??|°
-     color_used,?§?o
-            RET,\Х?\??                                                  */
+/*  ja:ビットマップのバイト数を求める
+          width,幅
+         height,高さ
+      bit_count,1ピクセルあたりのビット数
+    compression,圧縮形式
+     color_used,色数
+            RET,バイト数                                                    */
 #define bx_all_bytes(width,height,bit_count,compression,color_used)         \
             (bx_header_bytes(width,height,bit_count,compression,color_used) \
             +bx_image_bytes(width,height,bit_count,compression,color_used))
 
 
-/*  ja:RgbQuad1????ο?μ??e
-    bmih,BitmapInfoHeader1?????Υ??\?
-     RET,RgbQuad1????ο?                                                */
+/*  ja:RgbQuad構造体の数を求める
+    bmih,BitmapInfoHeader構造体へのポインタ
+     RET,RgbQuad構造体の数                                                  */
 #define bm_color_used(bmih) bx_color_used(bmih_get_width(bmih),             \
                         bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                         bmih_get_compression(bmih),bmih_get_color_used(bmih))
 
 
-/*  ja:RgbQuad1????????uint32?γ?ΥХ?\??μ??e
-    bmih,BitmapInfoHeader1?????Υ??\?
-     RET,\Х?\??                                                         */
+/*  ja:RgbQuad構造体またはguint32配列のバイト数を求める
+    bmih,BitmapInfoHeader構造体へのポインタ
+     RET,バイト数                                                           */
 #define bm_color_bytes(bmih) bx_color_bytes(bmih_get_width(bmih),           \
                         bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                         bmih_get_compression(bmih),bmih_get_color_used(bmih))
 
 
-/*  ja:BitmapInfoHeader1?????gbQuad1????????uint32?γ?ΥХ?\??μ??e
-    bmih,BitmapInfoHeader1?????Υ??\?
-     RET,\Х?\??                                                         */
+/*  ja:BitmapInfoHeader構造体とRgbQuad構造体またはguint32配列のバイト数を求める
+    bmih,BitmapInfoHeader構造体へのポインタ
+     RET,バイト数                                                           */
 #define bm_header_bytes(bmih)                                               \
-                bx_header_bytes(bmih_get_width(bmih),bmih_get_height(bmih), \
-                        bmih_get_bit_count(bmih),bmih_get_compression(bmih),\
-                                                    bmih_get_color_used(bmih))
+                    (bm_color_bytes(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE))
 
 
-/*  ja:1\?\?\Х?\??μ??e
-    bmih,BitmapInfoHeader1?????Υ??\?
-     RET,\Х?\??                                                         */
+/*  ja:1ラインのバイト数を求める
+    bmih,BitmapInfoHeader構造体へのポインタ
+     RET,バイト数                                                           */
 #define bm_line_bytes(bmih) bx_line_bytes(bmih_get_width(bmih),             \
                         bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                         bmih_get_compression(bmih),bmih_get_color_used(bmih))
 
 
-/*  ja:\?￣\≫\?!?\??ΥХ?\??μ??e
-    bmih,BitmapInfoHeader1?????Υ??\?
-     RET,\Х?\??                                                         */
+/*  ja:ピクセルデータのバイト数を求める
+    bmih,BitmapInfoHeader構造体へのポインタ
+     RET,バイト数                                                           */
 #define bm_image_bytes(bmih) bx_image_bytes(bmih_get_width(bmih),           \
                         bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                         bmih_get_compression(bmih),bmih_get_color_used(bmih))
 
 
-/*  ja:\?a??a?ΥХ?\??μ??e
-    bmih,BitmapInfoHeader1?????Υ??\?
-     RET,\Х?\??                                                         */
+/*  ja:ビットマップのバイト数を求める
+    bmih,BitmapInfoHeader構造体へのポインタ
+     RET,バイト数                                                           */
 #define bm_all_bytes(bmih) (bm_header_bytes(bmih)+bm_image_bytes(bmih))
 
 
 /******************************************************************************
 *                                                                             *
-* ja:\?a??a?uc´??                                                   *
+* ja:ビットマップ加工関数                                                     *
 *                                                                             *
 ******************************************************************************/
-/*  ja:?°?Υ??\????
-    count,μ???\?\???
-      rgb,\??\????1??\a?!
-    table,\??\?￢?Ω??                                                */
+/*  ja:任意のパレットを求める
+    count,求めるパレットの総数
+      rgb,パレットを格納するバッファ
+    table,パレット分布立方体                                                */
 void
 bitmap_make_palette (const gint  count,
                      RgbQuad    *rgb,
                      guint       table[32][32][32]);
 
 
-/*  ja:°μ??μ??\?a??a??，μ?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:圧縮されたビットマップを復元する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_expand (BitmapInfoHeader *bmih0,
                BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1???)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを1ビットビットマップに変換する(近似)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_approx_1 (BitmapInfoHeader *bmih0,
                  BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1???)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを4ビットビットマップに変換する(近似)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_approx_4 (BitmapInfoHeader *bmih0,
                  BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1???)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを8ビットビットマップに変換する(近似)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_approx_8 (BitmapInfoHeader *bmih0,
                  BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1???3?¶)
-    pbmih0,，μ?Υ?a??a??Υ??\?
-    pbmih1,?・?・??\?a??a??Υ??\?                                   */
+/*  ja:フルカラー/32ビットビットマップを1ビットビットマップに変換する(誤差拡散)
+    pbmih0,元のビットマップへのポインタ
+    pbmih1,新しいビットマップへのポインタ                                   */
 void
 bitmap_diffuse_1 (BitmapInfoHeader *bmih0,
                   BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1???3?¶)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを4ビットビットマップに変換する(誤差拡散)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_diffuse_4 (BitmapInfoHeader *bmih0,
                   BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1???3?¶)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを8ビットビットマップに変換する(誤差拡散)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_diffuse_8 (BitmapInfoHeader *bmih0,
                   BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1??±°e)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを1ビットビットマップに変換する(識域)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_order_1 (BitmapInfoHeader *bmih0,
                 BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1??±°e)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを4ビットビットマップに変換する(識域)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_order_4 (BitmapInfoHeader *bmih0,
                 BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a???a??a??a???1?1??±°e)
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを8ビットビットマップに変換する(識域)
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_order_8 (BitmapInfoHeader *bmih0,
                 BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?/32\?a??a??a??\?a??a??a???1?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラー/32ビットビットマップを16ビットビットマップに変換する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_convert_16 (BitmapInfoHeader *bmih0,
                    BitmapInfoHeader *bmih1);
 
 
-/*  ja:\?a??a??\?\?\?a??a???1?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:ビットマップをフルカラービットマップに変換する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_convert_24 (BitmapInfoHeader *bmih0,
                    BitmapInfoHeader *bmih1);
 
 
-/*  ja:32\?a??a??a??\?\?\?a??a???1?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?
-      rgb,?・?§                                                            */
+/*  ja:32ビットビットマップをフルカラービットマップに変換する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ
+      rgb,背景色                                                            */
 void
 bitmap_convert_24_with_bgcolor (BitmapInfoHeader *bmih0,
                                 BitmapInfoHeader *bmih1,
                                 RgbQuad          *rgb);
 
 
-/*  ja:\?a??a??\?a??a??a???1?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:ビットマップを32ビットビットマップに変換する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_convert_32 (BitmapInfoHeader *bmih0,
                    BitmapInfoHeader *bmih1);
 
 
-/*  ja:1\?a??a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:1ビットビットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom_1 (BitmapInfoHeader *bmih0,
                BitmapInfoHeader *bmih1);
 
 
-/*  ja:4\?a??a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:4ビットビットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom_4 (BitmapInfoHeader *bmih0,
                BitmapInfoHeader *bmih1);
 
 
-/*  ja:8\?a??a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:8ビットビットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom_8 (BitmapInfoHeader *bmih0,
                BitmapInfoHeader *bmih1);
 
 
-/*  ja:16\?a??a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:16ビットビットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom_16 (BitmapInfoHeader *bmih0,
                 BitmapInfoHeader *bmih1);
 
 
-/*  ja:\??\?\?a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:フルカラービットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom_24 (BitmapInfoHeader *bmih0,
                 BitmapInfoHeader *bmih1);
 
 
-/*  ja:32\?a??a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:32ビットビットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom_32 (BitmapInfoHeader *bmih0,
                 BitmapInfoHeader *bmih1);
 
 
-/*  ja:\?a??a?Υμ\?\o??11?1?e
-    bmih0,，μ?Υ?a??a??Υ??\?
-    bmih1,?・?・??\?a??a??Υ??\?                                    */
+/*  ja:ビットマップのサイズを変更する
+    bmih0,元のビットマップへのポインタ
+    bmih1,新しいビットマップへのポインタ                                    */
 void
 bitmap_zoom (BitmapInfoHeader *bmih0,
              BitmapInfoHeader *bmih1);
 
 
-/*  ja:32\?a??a??a??????≫?e
-        bmih0,2?????\a??a??Υ??\?
-        bmih1,?夋???\a??a??Υ??\?
-    x_overlay,Xo?，
-    y_overlay,Xo?，
-        trans,?2?,TRUE:?，?ALSE:?，?                                 */
+/*  ja:32ビットビットマップを重ね合わせる
+        bmih0,下になるビットマップへのポインタ
+        bmih1,上になるビットマップへのポインタ
+    x_overlay,X座標
+    y_overlay,X座標
+        trans,透過色,TRUE:有効,FALSE:無効                                   */
 void
 bitmap_overlay_32 (BitmapInfoHeader *bmih0,
                    BitmapInfoHeader *bmih1,
