@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,21 @@
 #include "avifmt.h"
 #include <gdk/gdkkeysyms.h>
 #include "misc/fileio.h"
-#include "misc/profile.h"
+#ifdef W32CODECDIR
+# include "misc/profile.h"
+#endif /* W32CODECDIR */
+
+
+#ifdef G_OS_WIN32
+# define icm_send_driver_message(icm_object,msg,lParam1,lParam2)            \
+    SendDriverMessage(((IcmObject *)(icm_object))->hDrvr,msg,lParam1,lParam2)
+#endif /* G_OS_WIN32 */
+#ifdef W32CODECDIR
+# define icm_send_driver_message(icm_object,msg,lParam1,lParam2)            \
+            ((IcmObject *)(icm_object))->icm_driver->DriverProc             \
+            (((IcmObject *)(icm_object))->dwDriverId,                       \
+            ((IcmObject *)(icm_object))->icm_driver->hLib,msg,lParam1,lParam2)
+#endif /* W32CODECDIR */
 
 
 /******************************************************************************
@@ -32,7 +46,7 @@ static GHashTable *ghash_handler = NULL, *ghash_path = NULL;
 
 
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-/*static guint32
+static guint32
 icm_handler_tolower (const guint32 key)
 {
   gint i;
@@ -44,10 +58,10 @@ icm_handler_tolower (const guint32 key)
       a |= (guchar)g_ascii_tolower ((gchar)(key >> i));
     }
   return a;
-}*/
+}
 
 
-static guint32
+/*static guint32
 icm_handler_toupper (const guint32 key)
 {
   gint i;
@@ -59,13 +73,13 @@ icm_handler_toupper (const guint32 key)
       a |= (guchar)g_ascii_toupper ((gchar)(key >> i));
     }
   return a;
-}
+}*/
 
 
 static guint
 icm_hash_handler (gconstpointer key)
 {
-  return icm_handler_toupper ((const guint32)key);
+  return icm_handler_tolower ((const guint32)key);
 }
 
 
@@ -73,8 +87,8 @@ static gboolean
 icm_equal_handler (gconstpointer a,
                    gconstpointer b)
 {
-  return icm_handler_toupper ((const guint32)a)
-                                    == icm_handler_toupper ((const guint32)b);
+  return icm_handler_tolower ((const guint32)a)
+                                    == icm_handler_tolower ((const guint32)b);
 }
 
 
@@ -113,11 +127,7 @@ icm_init_callback (gpointer key,
   if (g_hash_table_lookup_extended ((GHashTable *)ghash, key,
                                                     &orig_key, &orig_value))
     {
-# ifdef G_OS_WIN32
-      LPTSTR path;
-# else /* not G_OS_WIN32 */
       gchar *path;
-# endif /* not G_OS_WIN32 */
       guint32 handler;
       IcmDriver *icm_driver;
 
@@ -127,34 +137,40 @@ icm_init_callback (gpointer key,
       if (!icm_driver)
         {
           /* ja:新規ロード */
-          if ((icm_driver = g_malloc0 (sizeof (IcmDriver)))
 # ifdef G_OS_WIN32
-                    && (icm_driver->hLib = LoadLibrary (path))
-                    && ((FARPROC)icm_driver->DriverProc
-                            = GetProcAddress (icm_driver->hLib, "DriverProc")))
+          icm_driver = g_malloc0 (sizeof (IcmDriver));
+          if (icm_driver)
+            {
+              icm_driver->lpszName = g_utf8_to_utf16 (path, -1,
+                                                            NULL, NULL, NULL);
+              g_hash_table_insert (ghash_path, path, icm_driver);
+            }
+          else
+            {
+              g_free (path);
+            }
 # else /* not G_OS_WIN32 */
+          if ((icm_driver = g_malloc0 (sizeof (IcmDriver)))
                     && (icm_driver->hLib = peimage_create())
                     && peimage_load (icm_driver->hLib, path)
                     && (icm_driver->DriverProc
                         = peimage_resolve (icm_driver->hLib, "DriverProc")))
-# endif /* not G_OS_WIN32 */
             {
               icm_driver->DriverProc (0, icm_driver->hLib, DRV_LOAD, 0, 0);
               icm_driver->DriverProc (0, icm_driver->hLib, DRV_ENABLE, 0, 0);
+              icm_driver->lOpenID = icm_driver->DriverProc
+                                (0, icm_driver->hLib, DRV_OPEN, (LONG)L"", 0);
               g_hash_table_insert (ghash_path, path, icm_driver);
             }
           else
             {
               if (icm_driver->hLib)
-# ifdef G_OS_WIN32
-                FreeLibrary (icm_driver->hLib);
-# else /* not G_OS_WIN32 */
                 peimage_destroy (icm_driver->hLib);
-# endif /* not G_OS_WIN32 */
               g_free (icm_driver);
               g_free (path);
               icm_driver = NULL;
             }
+# endif /* not G_OS_WIN32 */
         }
       if (icm_driver)   /* ja:登録 */
         g_hash_table_insert (ghash_handler, (gpointer)handler, icm_driver);
@@ -172,7 +188,7 @@ icm_init (void)
   GHashTable *ghash;
 #endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
 #ifdef G_OS_WIN32
-  DWORD dwIndex, dwSize, dwType, dwValue;
+  DWORD dwIndex, dwName, dwType, dwValue;
   HKEY hKey;
   TCHAR szName[MAX_PATH], szValue[MAX_PATH];
 #endif /* G_OS_WIN32 */
@@ -188,26 +204,27 @@ icm_init (void)
         _T("Software\\Microsoft\\Windows NT\\CurrentVersion\\Drivers32"), 0,
                                     KEY_QUERY_VALUE, &hKey) != ERROR_SUCCESS)
     return FALSE;
-  for (dwIndex = 0; dwSize = MAX_PATH, dwValue = MAX_PATH,
-            RegEnumValue (hKey, dwIndex, szName, &dwSize, NULL,
+  for (dwIndex = 0; dwName = MAX_PATH, dwValue = MAX_PATH,
+            RegEnumValue (hKey, dwIndex, szName, &dwName, NULL,
             &dwType, (LPBYTE)szValue, &dwValue) == ERROR_SUCCESS; dwIndex++)
-    if (dwType == REG_SZ && (TCHAR)CharUpper((LPTSTR)szName[0]) == 'V'
-                         && (TCHAR)CharUpper((LPTSTR)szName[1]) == 'I'
-                         && (TCHAR)CharUpper((LPTSTR)szName[2]) == 'D'
-                         && (TCHAR)CharUpper((LPTSTR)szName[3]) == 'C'
-                         && (TCHAR)CharUpper((LPTSTR)szName[4]) == '.')
+    if (dwType == REG_SZ && (TCHAR)CharUpper ((LPTSTR)szName[0]) == 'V'
+                         && (TCHAR)CharUpper ((LPTSTR)szName[1]) == 'I'
+                         && (TCHAR)CharUpper ((LPTSTR)szName[2]) == 'D'
+                         && (TCHAR)CharUpper ((LPTSTR)szName[3]) == 'C'
+                         && (TCHAR)CharUpper ((LPTSTR)szName[4]) == '.')
       {
         gchar *key, *value;
         guint32 handler;
 
 # ifdef UNICODE
         key = g_utf8_from_utf16 (szName + 5, -1, NULL, NULL, NULL);
-        value = g_memdup (szValue, lstrlen (szValue) * sizeof (TCHAR));
+        value = g_utf8_from_utf16 (szName, -1, NULL, NULL, NULL);
 # else /* not UNICODE */
         key = g_locale_to_utf8 (szName + 5, -1, NULL, NULL, NULL);
-        value = g_strdup (szValue);
+        value = g_locale_to_utf8 (szName, -1, NULL, NULL, NULL);
 # endif /* not UNICODE */
-        if (key && value && (handler = icm_string_to_handler (key)) != 0)
+        if (key && value && (handler
+                    = icm_handler_tolower (icm_string_to_handler (key))) != 0)
           g_hash_table_insert (ghash, (gpointer)handler, value);
         g_free (key);
       }
@@ -231,14 +248,16 @@ icm_init (void)
               guint32 handler;
 
               key = g_list_nth_data (glist, i);
-              if ((handler = icm_string_to_handler (key)) != 0
+              if ((handler
+                    = icm_handler_tolower (icm_string_to_handler (key))) != 0
                         && (value = profile_get_string (profile, "vidc", key)))
                 {
                   gchar *path;
 
                   path = g_hash_table_lookup (ghash, (gconstpointer)handler);
                   g_free (path);
-                  if (value[0] == '\0' || value[g_strlen(value) - 1] == '/')
+                  if (value[0] == '\0'
+                            || value[g_strlen(value) - 1] == G_DIR_SEPARATOR)
                     {
                       if (path)
                         g_hash_table_remove (ghash, (gconstpointer)handler);
@@ -254,7 +273,8 @@ icm_init (void)
                         }
                       else
                         {
-                          path = g_strconcat (W32CODECDIR, "/", tmp, NULL);
+                          path = g_strconcat (W32CODECDIR,
+                                                G_DIR_SEPARATOR_S, tmp, NULL);
                           g_free (tmp);
                         }
                       tmp = path;
@@ -297,12 +317,14 @@ icm_exit_callback (gpointer key,
       IcmDriver *icm_driver;
 
       icm_driver = orig_value;
-      icm_driver->DriverProc (0, icm_driver->hLib, DRV_DISABLE, 0, 0);
-      icm_driver->DriverProc (0, icm_driver->hLib, DRV_FREE, 0, 0);
 #ifdef G_OS_WIN32
-      FreeLibrary (icm_driver->hLib);
+      g_free (icm_driver->lpszName);
 #endif /* G_OS_WIN32 */
 #ifdef W32CODECDIR
+      icm_driver->DriverProc (icm_driver->lOpenID,
+                                            icm_driver->hLib, DRV_CLOSE, 0, 0);
+      icm_driver->DriverProc (0, icm_driver->hLib, DRV_DISABLE, 0, 0);
+      icm_driver->DriverProc (0, icm_driver->hLib, DRV_FREE, 0, 0);
       peimage_destroy (icm_driver->hLib);
 #endif /* W32CODECDIR */
       g_free (icm_driver);
@@ -470,10 +492,17 @@ icm_open (const guint32 handler, const gint mode)
           case ICM_MODE_DRAW:
             icopen.dwFlags = ICMODE_DRAW;
         }
+# ifdef G_OS_WIN32
+      icm_object->hDrvr = OpenDriver (icm_object->icm_driver->lpszName, NULL,
+                                                                (LONG)&icopen);
+    }
+  if (!icm_object->hDrvr)
+# else /* not G_OS_WIN32 */
       icm_object->dwDriverId = icm_object->icm_driver->DriverProc
-                (1, icm_object->icm_driver->hLib, DRV_OPEN, 0, (LONG)&icopen);
+                (0, icm_object->icm_driver->hLib, DRV_OPEN, 0, (LONG)&icopen);
     }
   if (icm_object->dwDriverId == 0)
+# endif /* not G_OS_WIN32 */
     {
       g_free (icm_object);
       icm_object = NULL;
@@ -493,10 +522,13 @@ icm_close (IcmObject *icm_object)
 {
   if (icm_object && --(icm_object->counter) <= 0)
     {
-#if defined (G_OS_WIN32) || defined (W32CODECDIR)
+#ifdef G_OS_WIN32
+      CloseDriver (icm_object->hDrvr, 0, 0);
+#endif /* G_OS_WIN32 */
+#ifdef W32CODECDIR
       icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
                                 icm_object->icm_driver->hLib, DRV_CLOSE, 0, 0);
-#endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
+#endif /* W32CODECDIR */
       g_free (icm_object->bmih_in);
       g_free (icm_object->bmih_out);
       g_free (icm_object->prev);
@@ -521,15 +553,15 @@ icm_get_info (IcmObject *icm_object,
     return FALSE;
   g_memset (&icinfo, 0, sizeof (ICINFO));
   icinfo.dwSize = sizeof (ICINFO);
-  icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                    icm_object->icm_driver->hLib, ICM_GETINFO,
+  icm_send_driver_message (icm_object, ICM_GETINFO,
                                             (LONG)&icinfo, sizeof (ICINFO));
-  icm_info->name = g_utf16_to_utf8 (icinfo.szName, -1, NULL, NULL, NULL);
+  icm_info->name = g_utf16_to_utf8 (icinfo.szDescription, -1,
+                                                            NULL, NULL, NULL);
   icm_info->handler = icm_get_handler (icm_object); /* icinfo.fccHandler */
   icm_info->flags = 0;
   if (icinfo.dwFlags & VIDCF_QUALITY)
     icm_info->flags = ICM_FLAG_QUALITY;
-  if (icinfo.dwFlags&VIDCF_CRUNCH)
+  if (icinfo.dwFlags & VIDCF_CRUNCH)
     icm_info->flags |= ICM_FLAG_CRUNCH;
   if (icinfo.dwFlags & VIDCF_TEMPORAL)
     icm_info->flags |= ICM_FLAG_TEMPORAL;
@@ -561,9 +593,8 @@ icm_get_state_size (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   return icm_object
-                && icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                            icm_object->icm_driver->hLib, ICM_GETSTATE, 0, 0);
-#else /* notdefined (G_OS_WIN32) || defined (W32CODECDIR) */
+                ? icm_send_driver_message (icm_object, ICM_GETSTATE, 0, 0) : 0;
+#else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return 0;
 #endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
 }
@@ -581,8 +612,7 @@ icm_get_state (IcmObject *icm_object,
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   return icm_object && param && param_size > 0
-                && icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                    icm_object->icm_driver->hLib, ICM_GETSTATE,
+                        && icm_send_driver_message (icm_object, ICM_GETSTATE,
                                         (LPARAM)param, param_size) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
@@ -603,8 +633,7 @@ icm_set_state (IcmObject *icm_object,
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   if (icm_object && param && param_size > 0)
     {
-      icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                    icm_object->icm_driver->hLib, ICM_SETSTATE,
+      icm_send_driver_message (icm_object, ICM_SETSTATE,
                                                     (LPARAM)param, param_size);
       return TRUE;
     }
@@ -625,8 +654,7 @@ gboolean
 icm_is_dialog_about (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  return icm_object && icm_object->icm_driver->DriverProc
-                        (icm_object->dwDriverId, icm_object->icm_driver->hLib,
+  return icm_object && icm_send_driver_message (icm_object,
                                                 ICM_ABOUT, -1, 0) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
@@ -641,8 +669,7 @@ icm_dialog_about (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   if (icm_object)
-    icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                icm_object->icm_driver->hLib, ICM_ABOUT, 0, 0);
+    icm_send_driver_message (icm_object, ICM_ABOUT, 0, 0);
 #endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
 }
 
@@ -654,8 +681,7 @@ gboolean
 icm_is_dialog_configure (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  return icm_object && icm_object->icm_driver->DriverProc
-                        (icm_object->dwDriverId, icm_object->icm_driver->hLib,
+  return icm_object && icm_send_driver_message (icm_object,
                                             ICM_CONFIGURE, -1, 0) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
@@ -670,8 +696,7 @@ icm_dialog_configure (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   if (icm_object)
-    icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                            icm_object->icm_driver->hLib, ICM_CONFIGURE, 0, 0);
+    icm_send_driver_message (icm_object, ICM_CONFIGURE, 0, 0);
 #endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
 }
 
@@ -690,9 +715,7 @@ icm_compress_query (IcmObject              *icm_object,
                     const BitmapInfoHeader *bmih)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  return icm_object && bmih
-                && icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                icm_object->icm_driver->hLib,
+  return icm_object && bmih && icm_send_driver_message (icm_object,
                                 ICM_COMPRESS_QUERY, (LONG)bmih, 0) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
@@ -707,15 +730,22 @@ gint
 icm_compress_get_default_key_frame (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  DWORD dwKeyFrame;
+  IcmInfo icm_info;
 
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                icm_object->icm_driver->hLib,
-                ICM_GETDEFAULTKEYFRAMERATE, (LONG)&dwKeyFrame, 0) == ICERR_OK
-                                                            ? dwKeyFrame : 0;
-#else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
+  if (icm_object && icm_get_info (icm_object, &icm_info))
+    {
+      g_free (icm_info.name);
+      if ((icm_info.flags & ICM_FLAG_TEMPORAL) != 0)
+        {
+          DWORD dwKeyFrame;
+
+          if (icm_send_driver_message (icm_object,
+                ICM_GETDEFAULTKEYFRAMERATE, (LONG)&dwKeyFrame, 0) == ICERR_OK)
+            return dwKeyFrame;
+        }
+    }
+#endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return 0;
-#endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
 }
 
 
@@ -726,15 +756,23 @@ gint
 icm_compress_get_default_quality (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  DWORD dwQuality;
+  IcmInfo icm_info;
 
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                        icm_object->icm_driver->hLib,
-                        ICM_GETDEFAULTQUALITY, (LONG)&dwQuality, 0) == ICERR_OK
-                                                            ? dwQuality : -1;
-#else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
+  if (icm_object && icm_get_info (icm_object, &icm_info)
+                                && (icm_info.flags & ICM_FLAG_QUALITY) != 0)
+    {
+      g_free (icm_info.name);
+      if ((icm_info.flags & ICM_FLAG_QUALITY) != 0)
+        {
+          DWORD dwQuality;
+
+          if (icm_send_driver_message (icm_object,
+                    ICM_GETDEFAULTQUALITY, (LONG)&dwQuality, 0) == ICERR_OK)
+            return dwQuality;
+        }
+    }
+#endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return -1;
-#endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
 }
 
 
@@ -749,8 +787,7 @@ icm_compress_get_format_size (IcmObject              *icm_object,
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   if (!icm_object || !bmih)
     return 0;
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                    icm_object->icm_driver->hLib,
+  return icm_send_driver_message (icm_object,
                                     ICM_COMPRESS_GET_FORMAT, (LONG)bmih, 0);
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return 0;
@@ -770,8 +807,7 @@ icm_compress_get_format (IcmObject              *icm_object,
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   return icm_object && bmih_in && bmih_out
-        && icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-        icm_object->icm_driver->hLib,
+        && icm_send_driver_message (icm_object,
         ICM_COMPRESS_GET_FORMAT, (LONG)bmih_in, (LONG)bmih_out) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
@@ -787,8 +823,7 @@ icm_compress_get_size (IcmObject *icm_object)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   return icm_object && icm_object->bmih_in && icm_object->bmih_out
-                ? icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                    icm_object->icm_driver->hLib, ICM_COMPRESS_GET_SIZE,
+                ? icm_send_driver_message (icm_object, ICM_COMPRESS_GET_SIZE,
                     (LONG)icm_object->bmih_in, (LONG)icm_object->bmih_out) : 0;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return 0;
@@ -801,7 +836,7 @@ icm_compress_get_size (IcmObject *icm_object)
        bmih_in,圧縮するビットマップのヘッタ
       bmih_out,圧縮されたビットマップのヘッタ
      bmih_size,圧縮されたビットマップのヘッタのサイズ
-     key_frame,周期(0:すべてキーフレーム)
+     key_frame,周期(1:すべてキーフレーム,2～:周期,0:デフォルト)
        quality,品質0～10000,-1:デフォルト
      data_rate,1秒間のサイズ(0:無効)
           rate,レート
@@ -820,10 +855,10 @@ icm_compress_frames_info (IcmObject              *icm_object,
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   IcmInfo icm_info;
-  ICCOMPRESSFRAMES iccompressframes;
 
-  if (!icm_object || !icm_get_info(icm_object,&icm_info))
+  if (!icm_object || !icm_get_info (icm_object, &icm_info))
     return FALSE;
+  g_free (icm_info.name);
   if (bmih_in)
     {
       g_free (icm_object->bmih_in);
@@ -853,20 +888,21 @@ icm_compress_frames_info (IcmObject              *icm_object,
     icm_object->quality = icm_compress_get_default_quality (icm_object);
   if (icm_object->quality < 0 && 10000 < icm_object->quality)
     icm_object->quality = 0;
+  icm_object->key_frame = key_frame >= 1 ? key_frame
+                    : MAX (icm_compress_get_default_key_frame (icm_object), 1);
   if (icm_info.flags & ICM_FLAG_COMPRESSFRAMES)
     {
+      ICCOMPRESSFRAMES iccompressframes;
+
       g_memset (&iccompressframes, 0, sizeof (ICCOMPRESSFRAMES));
       (BitmapInfoHeader *)iccompressframes.lpbiOutput = icm_object->bmih_out;
       (BitmapInfoHeader *)iccompressframes.lpbiInput = icm_object->bmih_in;
-      iccompressframes.lFrameCount = icm_object->frame;
       iccompressframes.lQuality = icm_object->quality;
       iccompressframes.lDataRate = data_rate > 0 ? data_rate : G_MAXINT;
-      iccompressframes.lKeyRate = key_frame >= 1 ? key_frame
-                    : MAX (icm_compress_get_default_key_frame (icm_object), 1);
+      iccompressframes.lKeyRate = icm_object->key_frame;
       iccompressframes.dwRate = MAX (rate, 1);
       iccompressframes.dwScale = MAX (scale, 1);
-      icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                        icm_object->icm_driver->hLib, ICM_COMPRESS_FRAMES_INFO,
+      icm_send_driver_message (icm_object, ICM_COMPRESS_FRAMES_INFO,
                         (LONG)&iccompressframes, sizeof (ICCOMPRESSFRAMES));
     }
   return TRUE;
@@ -897,12 +933,12 @@ icm_compress_begin (IcmObject *icm_object)
           return FALSE;
         }
     }
-  if (icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-            icm_object->icm_driver->hLib, ICM_COMPRESS_BEGIN,
+  if (icm_send_driver_message (icm_object, ICM_COMPRESS_BEGIN,
             (LONG)icm_object->bmih_in, (LONG)icm_object->bmih_out) == ICERR_OK
                 && (icm_object->size = icm_compress_get_size (icm_object)) > 0)
     {
       icm_object->frame = 0;
+      icm_object->key_point = 0;
       icm_object->prev_size = bm_image_bytes (icm_object->bmih_in);
       icm_object->prev = g_malloc (icm_object->prev_size);
       return TRUE;
@@ -923,8 +959,8 @@ icm_compress_end (IcmObject *icm_object)
     return FALSE;
   g_free (icm_object->prev);
   icm_object->prev = NULL;
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-            icm_object->icm_driver->hLib, ICM_COMPRESS_END, 0, 0) == ICERR_OK;
+  return icm_send_driver_message (icm_object,
+                                        ICM_COMPRESS_END, 0, 0) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
 #endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
@@ -951,7 +987,8 @@ icm_compress (IcmObject *icm_object,
                             || !icm_object->bmih_in || !icm_object->bmih_out
                             || !icm_object->prev || icm_object->prev_size <= 0)
     return 0;
-  iccompress.dwFlags = ICCOMPRESS_KEYFRAME;
+  iccompress.dwFlags = icm_object->key_point % icm_object->key_frame == 0
+                                                    ? ICCOMPRESS_KEYFRAME : 0;
   (BitmapInfoHeader *)iccompress.lpbiOutput = icm_object->bmih_out;
   iccompress.lpOutput = output;
   (BitmapInfoHeader *)iccompress.lpbiInput = icm_object->bmih_in;
@@ -971,15 +1008,18 @@ icm_compress (IcmObject *icm_object,
       iccompress.lpbiPrev = NULL;
       iccompress.lpPrev = NULL;
     }
-  if (icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-            icm_object->icm_driver->hLib,
+  if (icm_send_driver_message (icm_object,
             ICM_COMPRESS, (LONG)&iccompress, sizeof (ICCOMPRESS)) == ICERR_OK
             && bmih_get_size_image (icm_object->bmih_out) <= icm_object->size)
     {
       icm_object->frame++;
+      if ((dwFlags & AVI_IF_KEYFRAME) != 0)
+        icm_object->key_point = 1;
+      else
+        icm_object->key_point++;
       g_memmove (icm_object->prev, input, icm_object->prev_size);
       if (key_frame)
-        *key_frame = dwFlags & AVI_IF_KEYFRAME;
+        *key_frame = (dwFlags & AVI_IF_KEYFRAME) != 0;
       return bmih_get_size_image (icm_object->bmih_out);
     }
 #endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
@@ -1002,8 +1042,7 @@ icm_decompress_query (IcmObject              *icm_object,
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
   return icm_object && bmih
-                && icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                            icm_object->icm_driver->hLib,
+                && icm_send_driver_message (icm_object,
                             ICM_DECOMPRESS_QUERY, (LONG)bmih, 0) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
@@ -1020,11 +1059,32 @@ icm_decompress_get_format_size (IcmObject              *icm_object,
                                 const BitmapInfoHeader *bmih)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  if (!icm_object || !bmih)
-    return 0;
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                                    icm_object->icm_driver->hLib,
+  gsize bmih_size = 0;
+
+  if (icm_object && bmih)
+    {
+      bmih_size = icm_send_driver_message (icm_object,
                                     ICM_DECOMPRESS_GET_FORMAT, (LONG)bmih, 0);
+      if (bmih_size > 0)
+        {
+          BitmapInfoHeader *bmih_out;
+
+          bmih_out = g_malloc (bmih_size);
+          if (icm_send_driver_message (icm_object, ICM_DECOMPRESS_GET_FORMAT,
+                                    (LONG)bmih, (LONG)bmih_out) == ICERR_OK)
+            {
+              if (bmih_size == bmih_get_size (bmih_out)
+                        && bm_color_bytes (bmih) == bm_color_bytes (bmih_out))
+                bmih_size += bm_color_bytes (bmih_out);
+            }
+          else
+            {
+              bmih_size = 0;
+            }
+          g_free (bmih_out);
+        }
+    }
+  return bmih_size;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return 0;
 #endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
@@ -1042,13 +1102,26 @@ icm_decompress_get_format (IcmObject              *icm_object,
                            BitmapInfoHeader       *bmih_out)
 {
 #if defined (G_OS_WIN32) || defined (W32CODECDIR)
-  return icm_object && bmih_in && bmih_out
-        && icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-        icm_object->icm_driver->hLib,
-        ICM_DECOMPRESS_GET_FORMAT, (LONG)bmih_in, (LONG)bmih_out) == ICERR_OK;
-#else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
+  if (icm_object && bmih_in && bmih_out
+        && icm_send_driver_message (icm_object,
+        ICM_DECOMPRESS_GET_FORMAT, (LONG)bmih_in, (LONG)bmih_out) == ICERR_OK)
+    {
+      gsize bmih_size;
+
+      bmih_size = icm_send_driver_message (icm_object,
+                                ICM_DECOMPRESS_GET_FORMAT, (LONG)bmih_in, 0);
+      if (bmih_size > 0)
+        {
+          if (bmih_size == bmih_get_size (bmih_out)
+                    && bm_color_bytes (bmih_in) == bm_color_bytes (bmih_out))
+            g_memmove ((guint8 *)bmih_out + bmih_get_size (bmih_out),
+                       (guint8 *)bmih_in  + bmih_get_size (bmih_in),
+                       bm_color_bytes (bmih_out));
+          return TRUE;
+        }
+    }
+#endif /* defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
-#endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
 }
 
 
@@ -1066,8 +1139,7 @@ icm_decompress_begin (IcmObject              *icm_object,
   icm_object->bmih_out
                 = g_malloc (icm_decompress_get_format_size (icm_object, bmih));
   if (!icm_decompress_get_format (icm_object,bmih, icm_object->bmih_out)
-                || icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                        icm_object->icm_driver->hLib,
+                || icm_send_driver_message (icm_object,
                         ICM_DECOMPRESS_BEGIN,
                         (LONG)bmih, (LONG)icm_object->bmih_out) != ICERR_OK)
     {
@@ -1096,8 +1168,8 @@ icm_decompress_end (IcmObject *icm_object)
   g_free (icm_object->bmih_out);
   icm_object->bmih_in = NULL;
   icm_object->bmih_out = NULL;
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-          icm_object->icm_driver->hLib, ICM_DECOMPRESS_END, 0, 0) == ICERR_OK;
+  return icm_send_driver_message (icm_object,
+                                        ICM_DECOMPRESS_END, 0, 0) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
 #endif /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
@@ -1138,8 +1210,7 @@ icm_decompress (IcmObject   *icm_object,
   (BitmapInfoHeader *)icdecompress.lpbiOutput = icm_object->bmih_out;
   icdecompress.lpOutput = output;
   icdecompress.ckid = 0;
-  return icm_object->icm_driver->DriverProc (icm_object->dwDriverId,
-                    icm_object->icm_driver->hLib, ICM_DECOMPRESS,
+  return icm_send_driver_message (icm_object, ICM_DECOMPRESS,
                     (LONG)&icdecompress, sizeof (ICDECOMPRESS)) == ICERR_OK;
 #else /* not defined (G_OS_WIN32) || defined (W32CODECDIR) */
   return FALSE;
