@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "thread.h"
 #include "toolbar.h"
 #include "misc/misc.h"
+#include "orz/orzmdi.h"
 
 
 /******************************************************************************
@@ -132,89 +133,84 @@ draw_caret (VmaidWindow *vmaid,
 void
 set_menu_bar (VmaidWindow *vmaid)
 {
-  GList *glist;
-  GtkWidget *menu;
-
   if (vmaid)
     {
-      gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU
-                (gtk_item_factory_get_widget (ifactory_menu, "<main>/Edit"))),
-                                                                        TRUE);
-      gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU
-                (gtk_item_factory_get_widget (ifactory_menu, "<main>/View"))),
-                                                                        TRUE);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Close"), TRUE);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Save As..."),
-                                    vmaid->avi_edit[0] || vmaid->avi_edit[1]);
-//    gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-//              "<main>/File/Preview..."),
-//                                  vmaid->avi_edit[0] || vmaid->avi_edit[1]);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Properties..."),
-                                    vmaid->avi_edit[0] || vmaid->avi_edit[1]);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/Undo"), vmaid->undo != NULL);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/Redo"), vmaid->redo != NULL);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/Cut"), vmaid->select.stream >= 0);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/Copy"), vmaid->select.stream >= 0);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/Delete"), vmaid->select.stream >= 0);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/Jump..."),
-                                    vmaid->avi_edit[0] || vmaid->avi_edit[1]);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/Edit/All"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
-      gtk_widget_set_sensitive (toolbar_items[ 2].item,TRUE);
-      gtk_widget_set_sensitive (toolbar_items[ 4].item,
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/file/close"), TRUE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/file/saveas"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/file/preview"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/file/properties"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/edit"), TRUE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/undo"), vmaid->undo != NULL);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/redo"), vmaid->redo != NULL);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/cut"), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/copy"), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/delete"), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/jump"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                "/edit/all"), vmaid->avi_edit[0] || vmaid->avi_edit[1]);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/view"), TRUE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "saveas")), TRUE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                                "properties")),
                     vmaid->avi_edit[0] != NULL || vmaid->avi_edit[1] != NULL);
-//    gtk_widget_set_sensitive (toolbar_items[ 6].item,
-//                  vmaid->avi_edit[0] != NULL || vmaid->avi_edit[1] != NULL);
-      gtk_widget_set_sensitive (toolbar_items[ 8].item,
-                                vmaid->select.stream >= 0);
-      gtk_widget_set_sensitive (toolbar_items[ 9].item,
-                                vmaid->select.stream >= 0);
-      gtk_widget_set_sensitive (toolbar_items[10].item, TRUE);
-      gtk_widget_set_sensitive (toolbar_items[11].item,
-                                vmaid->select.stream >= 0);
-      gtk_widget_set_sensitive (toolbar_items[13].item, vmaid->undo != NULL);
-      gtk_widget_set_sensitive (toolbar_items[14].item, vmaid->redo != NULL);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                                "preview")),
+                    vmaid->avi_edit[0] != NULL || vmaid->avi_edit[1] != NULL);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "cut")), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "copy")), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "paste")), TRUE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "delete")), vmaid->select.stream >= 0);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "undo")), vmaid->undo != NULL);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                        "redo")), vmaid->redo != NULL);
     }
   else
     {
-      gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU
-                (gtk_item_factory_get_widget (ifactory_menu, "<main>/Edit"))),
-                                                                        FALSE);
-      gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU
-                (gtk_item_factory_get_widget (ifactory_menu, "<main>/View"))),
-                                                                        FALSE);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Close"), FALSE);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Save As..."), FALSE);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Preview..."), FALSE);
-      gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                "<main>/File/Properties..."), FALSE);
-      gtk_widget_set_sensitive (toolbar_items[ 2].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[ 4].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[ 6].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[ 8].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[ 9].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[10].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[11].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[13].item, FALSE);
-      gtk_widget_set_sensitive (toolbar_items[14].item, FALSE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                                                "/file/close"), FALSE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                                                "/file/saveas"), FALSE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                                                "/file/preview"), FALSE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries,
+                                                "/file/properties"), FALSE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/edit"), FALSE);
+      gtk_widget_set_sensitive (misc_find_menu (menu_entries, "/view"), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "saveas")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "properties")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "preview")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "cut")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "copy")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "paste")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "delete")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "undo")), FALSE);
+      gtk_widget_set_sensitive (GTK_WIDGET (misc_find_toolbar (toolbar_entries,
+                                                        "redo")), FALSE);
     }
-  menu = gtk_item_factory_get_widget (ifactory_menu, "<main>/Window");
-  glist = gtk_container_children (GTK_CONTAINER (menu));
-  gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU (menu)),
-                                                    g_list_length (glist) > 0);
-  g_list_free (glist);
 }
 
 
@@ -520,11 +516,15 @@ history_operation (VmaidWindow    *vmaid,
 gboolean
 prompt_close (VmaidWindow *vmaid)
 {
-  if (vmaid->edit)
+  gint page;
+
+  page = orz_mdi_get_page_from_data (ORZ_MDI (mdi), vmaid);
+  if (orz_mdi_get_edited (ORZ_MDI (mdi), page))
     {
       gchar *text, *utf8str;
 
-      utf8str = g_filename_to_utf8 (vmaid->file, -1, NULL, NULL, NULL);
+      utf8str = g_filename_to_utf8
+                (orz_mdi_get_file (ORZ_MDI (mdi), page), -1, NULL, NULL, NULL);
       text = g_strdup_printf (_("File %s was edited.\nSave?"), utf8str);
       g_free (utf8str);
       switch (misc_message_box ("Video maid", text, 0,
@@ -532,35 +532,17 @@ prompt_close (VmaidWindow *vmaid)
         {
           case 0:
             {
-              GtkWidget *child;
+              gint page_num;
 
-              child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)));
-              if (g_object_get_data (G_OBJECT (child), "user_data") != vmaid)
-                {
-                  gint i;
-                  GList *glist;
-
-                  glist = gtk_container_children (GTK_CONTAINER (notebook));
-                  for (i = g_list_length (glist) - 1; i >= 0; i--)
-                    if (g_object_get_data (G_OBJECT (g_list_nth_data
-                                            (glist, i)), "user_data") == vmaid)
-                      {
-                        gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
-                                                                            i);
-                        break;
-                      }
-                  g_list_free (glist);
-                }
+              page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+              if (page != page_num)
+                gtk_notebook_set_current_page (GTK_NOTEBOOK (mdi), page);
               else
-                {
-                  child = NULL;
-                }
-              command_save (NULL, 0, NULL);
-              if (child)
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook), child));
-              if (!vmaid->edit)
+                page_num = -1;
+              command_save (NULL, NULL);
+              if (page_num >= 0)
+                gtk_notebook_set_current_page (GTK_NOTEBOOK (mdi), page_num);
+              if (!orz_mdi_get_edited (ORZ_MDI (mdi), page))
                 break;
             }
           case 2:
@@ -574,22 +556,6 @@ prompt_close (VmaidWindow *vmaid)
 }
 
 
-/*  ja:タブの編集済みマークの変更
-      vmaid,ウインドウ情報
-    is_edit,TRUE:編集済み,FALSE:未編集                                      */
-void
-vmaid_change_edit (VmaidWindow *vmaid,
-                   gboolean     is_edit)
-{
-  if (vmaid->edit != is_edit)
-    {
-      vmaid->edit = is_edit;
-      if (is_edit)
-        gtk_widget_show (vmaid->mark);
-      else
-        gtk_widget_hide (vmaid->mark);
-    }
-}
 /******************************************************************************
 *                                                                             *
 * ja:高レベル関数群                                                           *
