@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,40 +45,30 @@ signal_style_set (GtkWidget *widget,
 *                                                                             *
 ******************************************************************************/
 void
-signal_activate_menu_window (GtkWidget *widget,
-                             GtkWidget *child);
-
-
-void
-signal_activate_menu_history (GtkWidget *widget,
-                              gchar     *file);
-
-
-void
-signal_destroy_menu_history (GtkWidget *widget,
-                             gchar     *file);
+signal_activate_menu_history (GtkWidget   *widget,
+                              const gchar *file);
 
 
 /******************************************************************************
 *                                                                             *
-* ja:シグナル/イベント関数群(ノート)                                          *
+* ja:シグナル/イベント関数群(MDI)                                             *
 *                                                                             *
 ******************************************************************************/
 void
-signal_clicked (GtkWidget *widget,
-                GtkWidget *child);
+signal_close_window (GtkWidget   *widget,
+                     guint        page_num,
+                     VmaidWindow *vmaid);
 
 
 void
-signal_switch_page (GtkNotebook     *notebook,
-                    GtkNotebookPage *page,
-                    gint             page_num,
-                    gpointer         user_data);
+signal_switch_window (GtkWidget   *widget,
+                      guint        page_num,
+                      VmaidWindow *vmaid);
 
 
 void
-signal_destroy_notebook (GtkWidget *widget,
-                         gpointer   user_data);
+signal_destroy_mdi (GtkWidget *widget,
+                    gpointer   user_data);
 
 
 /******************************************************************************
@@ -144,11 +134,6 @@ signal_delete (GtkWidget *widget,
                gpointer   user_data);
 
 
-void
-signal_destroy_window (GtkWidget *widget,
-                       gpointer   user_data);
-
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
